package cz.zcu.kiv.cosi.parkoviste.brana;

import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import cz.zcu.kiv.cosi.parkoviste.konfigurace.IKonfigurace;
import cz.zcu.kiv.cosi.parkoviste.konfigurace.IZapVyp;
import cz.zcu.kiv.cosi.parkoviste.parkoviste.IPrijezdyOdjezdy;

public class Brana implements IZapVyp, IPrijeloOdjelo, Runnable, EventHandler {

	private IPrijezdyOdjezdy parkoviste;

	private int prijelo;
	private int odjelo;

	private boolean beziSimulace;
	private boolean parkovistePlne;

	private int frekvence;
	private static final int MS_ZA_MINUTU = 60000;

	/* souet 1, ppsti typu - auto, bus, tir */
	private static final double[] PPSTI_VOZIDEL = {0.5, 0.2, 0.3};

	private static final int AUTO = 0;
	private static final int BUS = 1;
	private static final int TIR = 2;
	
	public synchronized void setFrekvence(IKonfigurace konfigurace) {
		this.frekvence = konfigurace.getFrekvence();
	}
	
	public synchronized void setParkoviste(IPrijezdyOdjezdy parkoviste) {
		this.parkoviste = parkoviste;
	}

	public Brana() {
		System.out.println("Brana created...");
		this.prijelo = 0;
		this.odjelo = 0;

		setParkovisteVolne();
	}

	@Override
	public void vyp() {
		beziSimulace = false;
		println("Vypnuto");
	}

	@Override
	public void zap() {
		beziSimulace = true;
		(new Thread(this)).start();
		println("Zapnuto");
	}	

	private synchronized boolean isParkovistePlne(){
		return parkovistePlne;
	}

	private synchronized void setParkovistePlne(){
		parkovistePlne = true;
	}

	private synchronized void setParkovisteVolne(){
		parkovistePlne = false;
	}

	@Override
	public void run() {

		while(beziSimulace){

			try {
				Thread.sleep(Math.round((exponential(frekvence) * MS_ZA_MINUTU)));
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

			if(jePrijezd()){
				prijezd(typVozidla());
			}else{
				odjezd(typVozidla());
			}
		}
	}

	private void prijezd(int typ){

		if(isParkovistePlne()){
			return;
		}

		try {
			switch(typ){
			case AUTO: println("Prijezd auto."); parkoviste.prijeloAuto(); break;
			case BUS: println("Prijezd bus."); parkoviste.prijelBus(); break;
			case TIR: println("Prijezd tir."); parkoviste.prijelTir(); break;
			}

			prijelo++;
		} catch (Exception e) {
			//ignorujeme
		}
	}

	private void odjezd(int typ){

		try {
			switch(typ){
			case AUTO: println("Odjezd auto."); parkoviste.odjeloAuto(); break;
			case BUS: println("Odjezd bus."); parkoviste.odjelBus(); break;
			case TIR: println("Odjezd tir."); parkoviste.odjelTir(); break;
			}

			odjelo++;
			setParkovisteVolne();

		} catch (Exception e) {
			//ignorujeme
		}
	}

	public static double exponential(double lambda) {
		return -Math.log(1 - Math.random()) / lambda;
	}

	private static boolean jePrijezd(){

		if(Math.random() < 0.5){
			return true;
		}else{
			return false;
		}
	}

	private static int typVozidla(){

		double interval = 0;
		double ppst = Math.random();

		int vybranyTyp = 0;

		for(int i = 0; i < PPSTI_VOZIDEL.length; i++){
			//nastaveni hranice intervalu pro losovani
			interval += PPSTI_VOZIDEL[i];

			//pokud je generovane cislo z daneho intervalu
			if (ppst <= interval) {
				vybranyTyp = i;
				break;
			}
		}

		return vybranyTyp;
	}

	private void println(String output) {
		System.out.println("Brana: " + output);
	}

	@Override
	public int getOdjelo() {
		return odjelo;
	}

	@Override
	public int getPrijelo() {
		return prijelo;
	}

	@Override
	public void handleEvent(Event event) {
		if ("parkoviste/plne".equals(event.getTopic()))
			setParkovistePlne();
	}

}
